import time
from datetime import datetime
import json
from faker import Faker
from confluent_kafka import Producer


def generate_fake_userClick():
    """Generate fake userClick data."""
    fake = Faker()

    userClick_data = {
        "user_id": fake.uuid4(),
        "timestamp": fake.date_time_this_month(),
        "page_id": fake.random_number(digits=5),
        "action": fake.random_element(elements=("click", "hover", "scroll"))
    }
    # Convert datetime object to string representation
    userClick_data["timestamp"] = userClick_data["timestamp"].strftime("%Y-%m-%d %H:%M:%S")
    return userClick_data


def delivery_report(err, msg):
    """Delivery report callback called on producing completion or failure."""
    if err is not None:
        print(f'Message delivery failed: {err}')
    else:
        print(f'Message delivered to {msg.topic()} [{msg.partition()}]')

def produce_to_kafka(topic_name, bootstrap_servers='localhost:9092'):
    """Produce messages to a Kafka topic."""
    producer_conf = {
        'bootstrap.servers': bootstrap_servers,
        'client.id': 'python-producer'
    }

    # Create KafkaProducer instance
    producer = Producer(producer_conf)

    try:
        while True:
            # Generate fake userClick data
            userClick_msg = generate_fake_userClick()

            userClick_serialized_msg = json.dumps(userClick_msg).encode('utf-8')

            # Produce message to the topic
            producer.produce(topic_name, userClick_serialized_msg, callback=delivery_report)

            # Sleep for a while before producing the next message
            time.sleep(20)

    except KeyboardInterrupt:
        # Handle keyboard interrupt (Ctrl+C)
        print("KeyboardInterrupt: Stopping Kafka producer")
    finally:
        # Flush and close the producer
        producer.flush()
        # producer.close()

# Example usage:
if __name__ == "__main__":
    topic_name = "userClick_msg"
    produce_to_kafka(topic_name)